<?php

/**
 * Novalnet Callback Script for Commerce Kickstart
 *
 * NOTICE
 *
 * This script is used for real time capturing of parameters passed
 * from Novalnet AG after Payment processing of customers.
 *
 * This script is only free to the use for Merchants of Novalnet AG
 *
 * If you have found this script useful a small recommendation as well
 * as a comment on merchant form would be greatly appreciated.
 *
 * Please contact sales@novalnet.de for enquiry or info
 *
 * ABSTRACT: This script is called from Novalnet, as soon as a payment
 * done for payment methods, e.g. Prepayment, Invoice.
 * An email will be sent if an error occurs
 *
 *
 * @category   Novalnet
 * @package    Novalnet
 * @version    1.0
 * @copyright  Copyright (c) Novalnet AG. (https://www.novalnet.de)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @notice     1. This script must be placed in Ubercart root folder
 *                to avoid rewrite rules (mod_rewrite)
 *             2. You have to adapt the value of all the variables
 *                commented with 'adapt ...'
 *             3. Set $test/$debug to false for live system
 */
define('DRUPAL_ROOT', getcwd());
require_once 'includes/bootstrap.inc';
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
//(loads everything, but doesn't render anything)
require_once('includes/module.inc');

//Table Names
define('ORDER_TABLE', 'commerce_order');
define('MSG_TABLE', 'commerce_payment_transaction');

//Variable Settings
$debug =  False; //false|true; adapt: set to false for go-live
$test  =  False; //false|true; adapt: set to false for go-live
$lineBreak = empty($_SERVER['HTTP_HOST']) ? PHP_EOL : '<br />';
$addSubsequentTidToDb = true; //whether to add the new tid to db; adapt if necessary
// Order State/Status Settings
/*   Standard Types of Status:
  ~~~~~~~~~~~~~~~~~~~~~~~
  pending
  success
  failure

 */
$transationState = "success"; //Note: Indicates Payment accepted.
$orderStatus = "completed";
$request = $_REQUEST;
//adapt here if needed; Options are:
$aPaymentTypes = array(
        'commerce_novalnet_invoice'         => 'INVOICE_CREDIT',
        'commerce_novalnet_prepayment'      => 'INVOICE_CREDIT',
        'commerce_novalnet_paypal'          => 'PAYPAL');

$allowedPayment = array('commerce_novalnet_invoice', 'commerce_novalnet_prepayment', 'commerce_novalnet_paypal');

 //adapt here if needed; Options are:

//Security Setting; only this IP is allowed for call back script
$ipAllowed = '195.143.189.210'; //Novalnet IP, is a fixed value, DO NOT CHANGE!!!!!
//Reporting Email Addresses Settings
$shopInfo = 'Commerce Kickstart' . $lineBreak; //manditory;adapt for your need
$mailHost = 'Ihr Email Server'; //adapt
$mailPort = 25; //adapt
$emailFromAddr = 'Ihr Emailaddr'; //sender email addr., manditory, adapt it
$emailToAddr = 'Ihr Emailaddr'; //recipient email addr., manditory, adapt it
$emailSubject = 'Novalnet Callback Script Access Report'; //adapt if necessary;
$emailBody = ''; //Email text, adapt
$emailFromName = "Novalnet Test"; // Sender name, adapt
$emailToName = ""; // Recipient name, adapt
//Parameters Settings
$hParamsRequired = array(
    'vendor_id' => '',
    'tid' => '',
    'payment_type' => '',
    'status' => '',
    'amount' => '',
    'tid_payment' => '');

    if($request['payment_type'] == $aPaymentTypes['commerce_novalnet_paypal'])
    {
      unset($hParamsRequired['tid_payment']);
    }
ksort($hParamsRequired);


//Test Data Settings.
if ($test) {
    $emailFromName = "Novalnet Test"; // Sender name, adapt
    $emailToName = ""; // Recipient name, adapt
    $emailFromAddr = 'Ihr Emailaddr'; //manditory for test; adapt
    $emailToAddr = ''; //manditory for test; adapt
    $emailSubject = $emailSubject . ' - TEST'; //adapt
}
// ################### Main Prog. ##########################
try {
    //Check Params
    if (checkIP($request)) {
        if (checkParams($request)) {
            //Get Order ID and Set New Order Status
            getIncrementId($request);
        }
    }

    if (!$emailBody) {
        $emailBody .= 'Novalnet Callback Script called for StoreId Parameters: ' . print_r($request, true) . $lineBreak;
        $emailBody .= 'Novalnet callback succ. ' . $lineBreak;
        $emailBody .= 'Params: ' . print_r($request, true) . $lineBreak;
    }
} catch (Exception $e) {
    //$emailBody .= "Exception catched: $lineBreak\$e:".$e->getMessage().$lineBreak;
}

if ($emailBody) {
    if (!sendEmail($emailBody)) {
        if ($debug) {
            echo "Mailing failed!" . $lineBreak;
            echo "This mail text should be sent: " . $lineBreak;
            echo $emailBody;
        }
    }
}

// ############## Sub Routines #####################
/*Method to send the callback script result mail */
function sendEmail($emailBody) {
    global $lineBreak, $debug, $test, $emailFromAddr, $emailToAddr, $emailFromName, $emailToName, $emailSubject, $shopInfo, $mailHost, $mailPort;
    //Send Email.
    $emailBodyT = str_replace('<br />', PHP_EOL, $emailBody);

    ini_set('SMTP', $mailHost);
    ini_set('smtp_port', $mailPort);

    header('Content-Type: text/html; charset=iso-8859-1');
    $headers = 'From: ' . $emailFromAddr . "\r\n";

    try {
        if ($debug) {
            echo __FUNCTION__ . ': Sending Email suceeded!' . $lineBreak;
        }
        $sendmail = mail($emailToAddr, $emailSubject, $emailBodyT, $headers);
    } catch (Exception $e) {
        //if ($debug) {echo 'Email sending failed: '.$e->getMessage();}
        return false;
    }
    if ($debug) {
        echo 'This text has been sent:' . $lineBreak . $emailBody;
    }
    return true;
}

/*Method to validate the url parameters */
function checkParams($_request) {
    global $lineBreak, $hParamsRequired, $emailBody, $aPaymentTypes;
    $error = false;
    $emailBody = '';
    if (!$_request) {
        $emailBody .= 'Novalnet callback received. No params passed over!' . $lineBreak;
        return FALSE;
    }
    if (!isset($_request['payment_type'])) {
        $emailBody .= 'Novalnet callback received. But Param payment_type missing' . $lineBreak;
        return FALSE;
    }

   if ($hParamsRequired) {
        foreach ($hParamsRequired as $k => $v) {
            if (empty($_request[$k])) {
                $error = true;
                $emailBody .= 'Novalnet callback received. Required param (' . $k . ') missing!' . $lineBreak;
            }
        }
        if ($error) {
            return false;
        }
    }
    //Only Payment Type 'INVOICE_CREDIT' allowed; Otherwise you have to adapt the logic
    if (!in_array($_request['payment_type'], $aPaymentTypes)) {
        $emailBody .= "Novalnet callback received. Payment type [".$_request['payment_type'] ."] is mismatched!". $lineBreak;
        return FALSE;
    }


    if (!empty($_request['status']) and 100 != $_request['status']) {
        $emailBody .= 'Novalnet callback received. Status [' . $_request['status'] . '] is not valid: Only 100 is allowed.' . $lineBreak . $lineBreak . $lineBreak;
        return FALSE;
    }
    //Check the novalnet payment tid for order
   if($_request['payment_type'] != $aPaymentTypes['commerce_novalnet_paypal']) {

        if (strlen($_request['tid_payment']) != 17) {
            $emailBody .= 'Novalnet callback received. Invalid TID [' . $_request['tid_payment'] . '] for Order.' . $lineBreak . $lineBreak . $lineBreak;
            return FALSE;
        }

     }

    //Check the novalnet subsequent tid
    if (strlen($_request['tid']) != 17) {
         if($_request['payment_type'] != $aPaymentTypes['commerce_novalnet_paypal']) {
            $emailBody .= 'Novalnet callback received.  New TID[' . $_request['tid'] . ']  is not valid .' . $lineBreak . $lineBreak . $lineBreak;
        } else {
            $emailBody .= 'Novalnet callback received.  TID [' . $_request['tid'] . '] is not valid.' . $lineBreak . $lineBreak . $lineBreak;

       }
         return FALSE;
    }
   return true;
  }

/*Method to find the orderid and validate the order */
function getIncrementId($_request) {
    global $lineBreak, $tableOrderPayment, $tableOrder, $emailBody, $debug, $oDB, $orderDetails, $order,$aPaymentTypes,$allowedPayment;
    $orderDetails = array();
    $transDetails = array();

    //check amount
    $amount = $_request['amount'];
    $_amount = isset($order_total) ? $order_total * 100 : 0;

    if (!$amount || $amount < 0) {
        $emailBody .= "Novalnet callback received. The requested amount ($amount) must be greater than zero." . $lineBreak . $lineBreak;
        return FALSE;
    }
    $iOrderId = 0;
    $original_tid = ($_request['payment_type'] == $aPaymentTypes['commerce_novalnet_paypal'])? $_request['tid'] : $_request['tid_payment'];
    $mysql = db_query("SELECT * from {" . MSG_TABLE . "} WHERE message LIKE '%" . $original_tid . "%'")->fetchAll();

    $transDetails = array_pop($mysql);
    $order_no = (isset($_request['order_no']) && !empty($_request['order_no'])) ? $_request['order_no'] : '';

    if (!$transDetails) {

        $emailBody .= 'Novalnet callback received. But passed Payment TID [' . $original_tid . '] invalid for Order.';
        $emailBody .= $lineBreak . $lineBreak . $lineBreak;
        return false;
    } else {
        if ($order_no != '') {
            if ($transDetails->order_id == $order_no) {
                $commerce_orderId = $transDetails->order_id;
            } else {
                $emailBody .= 'Novalnet callback received. Request Order id / Transaction id is mismatch!';
                $emailBody .= $lineBreak . $lineBreak . $lineBreak;
                return false;
            }
        } else {
            $commerce_orderId = $transDetails->order_id;
        }
    }

    $orderDetails = commerce_order_load($commerce_orderId);
    $wrapper = entity_metadata_wrapper('commerce_order', $orderDetails);

    if (empty($commerce_orderId) or !$orderDetails) {
        $emailBody .= 'Novalnet callback received. Payment type is is mismatched!';
        $emailBody .= $lineBreak . $lineBreak . $lineBreak;
        return false;
    }

    $order_total = $wrapper->commerce_order_total->amount->value();
    $transDetails->order_total = $order_total;
    $order_price = $order_total / 100;
    $roundofprice = round($order_price, 2);
    $_amount = isset($roundofprice) ? $roundofprice : 0;


    $paymentType = $transDetails->payment_method;
        //Only Payment Type 'INVOICE_CREDIT' allowed; Otherwise you have to adapt the logic
        if((isset($allowedPayment) && in_array($paymentType, $allowedPayment)) && ($_request['payment_type'] != $aPaymentTypes[$paymentType])) {

    $emailBody .= "Novalnet callback received. Payment type (".$_request['payment_type'].") is not $aPaymentTypes[$paymentType]!$lineBreak$lineBreak";


     return false;
  }
    $arr = explode("|", $paymentType);
    $paymentType = $arr[0];
    if (!in_array(strtolower($paymentType), $allowedPayment)) {


       $emailBody .= 'Novalnet callback received. Payment type ('.$paymentType.') is mismatched!
' . $lineBreak . $lineBreak;
        return false;
    }
    if ($debug) {
        echo'Order Details:' . $lineBreak . $lineBreak . 'Order No: ' . $orderDetails->order_id . $lineBreak . 'Amount: ' . $_amount . ' ' . $transDetails->currency_code . '</pre>' . $lineBreak . $lineBreak . $lineBreak;
    }

    setOrderStatus($transDetails, $_request);
}

/*Method to update the order balance, order status and novalnet order comments */
function setOrderStatus($transDetails, $_request) {
    global $lineBreak, $createInvoice, $emailBody, $orderStatus, $transationState, $tableOrderPayment, $addSubsequentTidToDb, $oDB,$aPaymentTypes;
    $incrementId = $transDetails->order_id;
    if ($incrementId) {
        if ($transDetails->status != $transationState) {
            if($_request['payment_type'] == $aPaymentTypes['commerce_novalnet_paypal']) {
                  $emailBody .= 'Novalnet Callback Script executed successfully.Payment for order id: ' . $incrementId .' on '. date("Y-m-d H:i:s");
            } else {
                  $emailBody .=" Novalnet Callback Script executed successfully. The subsequent TID: (" . $_request["tid"] . ") on " . date("Y-m-d H:i:s");

             }
            $rec_time = time();
            if($_request['payment_type'] == $aPaymentTypes['commerce_novalnet_paypal']) {
                 $updatecomments = db_query("UPDATE {" . MSG_TABLE . "} set message = CONCAT(message,'<br>Novalnet Callback Script executed successfully on " . date("Y-m-d H:i:s") . "') , status = '" . $transationState . "', amount = " . $transDetails->order_total . " WHERE order_id=" . $incrementId);

            } else {
            $updatecomments = db_query("UPDATE {" . MSG_TABLE . "} set message = CONCAT(message,'<br>Novalnet Callback Script executed successfully. The subsequent TID: (" . $_request["tid"] . ") on " . date("Y-m-d H:i:s") . "') , status = '" . $transationState . "', amount = " . $transDetails->order_total . " WHERE order_id=" . $incrementId);
        }
        } else {
            $emailBody .= "Novalnet callback received. Callback Script executed already. Refer Order :" . $incrementId;
            return FALSE;
        }
    } else {
        $emailBody .= "Novalnet Callback: No order for Increment-ID $incrementId found.";
        return false;
    }
    return true;
}

/*Method to validate the the IP address */
function checkIP($request) {
    global $lineBreak, $ipAllowed, $test, $emailBody;
    $callerIp = $_SERVER['REMOTE_ADDR'];
    if (($ipAllowed != $callerIp) && !$test) {
      $emailBody .= 'Novalnet callback received. Unauthorised access from the IP [' . $callerIp . ']' . $lineBreak . $lineBreak;
      $emailBody .= 'Request Params: ' . print_r($request, true);
      return false;
    }
    return true;
}

?>
